/*! \mainpage THE LEO ROBOT PROJECT
 *
 * \section intro_sec Introduction
 *
 * After using an ATMega8 for some stupid 50 Hz Sine generation I looked for a project that would use it more efficiently. So I came to the idea to build a robot which has no special purpose but to improve my skills (an to get a much cooler guy with a robot you know *G*)

 In the following sites there is the documentation for the sourcecode. If you aren't used to programing you won't find it very useful...

 Maybe for some interest. The used programs:
  - Eagle: for drawing the scheme. The personal version is free and I like it much.
  - LTSpice: for simulating the receiver circuit of the IR-light (frequency "selective")
  - Doxygen: A wonderful documentation helper (otherwise this page wouldn't look so nice)
  - Quanta+: Mainly not for writing this page but for editing the sourcecode. It has some nice skripting features.
  - KDevelop: After some time I decidet to switch to a more reasonable text-editor *g*
  - avr-gcc: As Compiler for this robotprogram
  - minicom: A small terminal used by the station
  - Hermes: The terminal application for Psion
 */

/* FILES */
/*! \file robodoc.c
    \brief This file contains the documentation for the project
*/
/*! \file robot.c
    \brief The main code for the robot. Mostly the Interrupt-functions
*/
/*! \file basic.c
   \brief Basic datatypes and definitions
*/
/*! \file basic.h
   \brief Basic datatypes and definitions
*/
/*! \file robotu.c
    \brief Helper functions for robot.
*/

/*! \file robotsound.c
    \brief Contains the sound routines.

    This file is for using the "sound system".

    The speaker is at Pin 17 and uses the Timer2. Possible frequencies are from 18Hz to about 50kHz, but the high frequencies don't have a good quantisation. The tone is always a square wave.
*/

/*! \file robotdo.c
    \brief The do-functions of the robot.

    Every modus, besides the fast radar and ir-transmitting modes, has a function do_<modus>. These is called once every 0.5 ms or about 1900Hz (depending on the speed of them). Basic routines like decode_position() or drive() are called before the do-functions.
*/
/*! \file ir.c
    \brief Basic IR-routines. This file is shared with the station.
*/
/*! \file ir.h */
/*! \file robotdo.h */
/*! \file robotu.h */
/*! \file robotsound.h */

/* FUNCTIONS */

/*! \fn SIGNAL(SIG_OVERFLOW0)
    \brief This function is called by the Timer0 which is the general "tasking" clock of the robot.

    There are two modes of the Timer used: the "normal" timing where the prescaler divides the clock by 8 and the "fast" without prescaler. The fast mode is used for the radar and the transmitting and receiving IR-routines. While these are active the normal do_functions are not called. (IR needs a clock of about 36kHz).
*/

/*! \fn SIGNAL(SIG_ADC)
    \brief Here the results of the ADC are evaluated.

    This function is called when the ADC is ready. Which AD-Channel was used and for what use is determined by admodus. The ir_station modus is more complicated as it toggles the IR-LED four times and reinitializes the ADC. The last ADC-value is taken into ir_station_value.
*/

/*! \fn main()
    \brief The main program. Does some initialisation and keeps running an eternal loop
*/

/*! \fn void play(u16 Hz)
    \brief Plays a tone in Herz

    This function should not be invoced directly (just for testing purposes) because it overrides the normal sound and doesn't stop brawling.
    \param Hz
    \brief The tone in Herz. 0-17 for switching off.
*/

/*! \fn void playt(u08 tone)
    \brief Plays a tone described in start_sound(u08 *soundtostart, u08 override)
*/

/*! \fn void start_sound(u08 *soundtostart, u08 override)
    \brief With this function you can enable a sound (melodie).
    The sound bites are in program memory and have the following structure:

    [ticks, tone],[ticks,tone],...[0, 0].

    One tick is about 16ms and by setting tone to 0 you can make a pause. The tones start at a C [33Hz] and go up 7 octaves. tone = 60 is the standart pitch 440Hz max is 83.
    a value of 85 plaies rising tones. and 86 does the same downwards.
    \param soundtostart
    A pointer to the sound which is ended by 0,0
    \param override
    Set this to one if the sound should replace a playing sound.
*/

/*! \fn void playi()
    \brief This function is called from SIGNAL(SIG_OVERFLOW0)

    Here the sound is playied. It is called every 16,3ms which is the fastest Tick in a sound.
*/
/*! \def START_BEEP(nbeeps)
    \brief starts the beeping of nbeeps sounds. nbeeps should not be greater than 10
*/
/*! \fn void do_boredom
    \brief Behaviour for standard operation

    In normal operation the robot will drive around and do measure the distance with his radar.
*/

/*! \fn void do_sleep
    \brief Behaviour for sleeping

    When the surrounding light is too less the robot will fall in a sleep mode. It will snore a bit and remain quiet until the light gehts brighter.
*/
/*! \fn void do_empty
    \brief Behaviour for empty

    When the loading station is not reachable and the voltage sinks under 3,5V the robot goes to this modus. It will send three beeps out and tries to save energy. By putting hin to the loadingstation it will "wake up".
*/

/*! \fn void do_flee
    \brief Behaviour for flee

    When the robot stays around and someone pushes him he tries to resist first. After a while he flees and drives away.
*/

/* FUNCTIONS OF robotu.* */

/*! \fn extern s08 sine(u08 in);
    \brief Sine of in

    Calculates the sine. Input range is 0..255 which is mapped to -128..127 (0..2*Pi)
    This function is only available if compiled with #define SINUS. It will cost about 200bytes of flash.
*/

/*! \fn extern void decode_position();
    \brief Decodes the angle sensor

    This function is called every 0.5ms to decode the angle sensors. Every 0,13 s it updates the heading, posx and posy. Furthermore it  avoids overflows of positionl and positionr while saving the heading. That's also the cause for which you should not use direct assignments to l_to and r_to.
*/

/*! \fn extern void drive_to_pos(s16 l2,s16 r2);
   \brief gives a target for the robot

   With this function you can force the robot to go exactly l2 steps left and r2 steps right.
*/

/** \fn void drive()
    \brief This function calculates and sets the Driver-IC

    In this function the values l_to and r_to are used to drive the motors. Furthermore speed and direction are read to increment l_to and r_to. By doing this it prioritises the heading, so the robot will slow down if it cannot hold the direction.
*/

/*! \fn u08 atoi(u08 *str)
   \brief Converts a hexnumber stored in a string to a u08.
*/
/*! \fn u16 abs(s16 val)
   \brief returns the positive value of val
*/

/*! \fn void ir_int_send(u16 val)
   \brief sends an integer value via IR. The leading '0's are transmitted
*/

/* FUNCTIONS OF ir.c */

/*! \fn extern void start_ir_r();
    \brief starts the recieving of an IR Pulse.

    This will last maximum (irlength * irbittime / 36000) while nothing else will be computated.
    The result will be writte to irbuf
*/

/*! \fn extern void ir_send(u08 *irbuf_)
    \brief starts the sending of an IR Pulse.

    This will last (irbuf_[0] * irbittime /36000) while nothing else will be computated. The data is given in irbuf_
*/
/*! \fn u08 *val_to_ir(u08 dat)
   \brief Converts a byte-value to an ir-impuls stored in irdata
*/
/*! \fn u08 ir_to_val(irdatatype data)
   \brief Converts and returns an ir-impuls as value
*/
/*! \fn void do_ir()
   \brief The main Ir-Routine. Does sending, receiving and possible converting to a string.

   The results of receiving are stored in irbuf and irstring. The values pulsevalid and stringvalid return if any data was received.
*/
/*! \fn u08 ir_comp(irdatatype cmp_target, irdatatype read_ir)
   \brief Compares two IR pulses.

   This is used for ircont. The bigger the result the narrower the pulses.
   \param read_ir
   The received pulse.
   \param cmp_target
   The stored "and"-array. It is generated by the loading station.
*/
/*! \fn void ir_and(irdatatype res, irdatatype and2)
   \brief builds a mask for the ir-pulses. Only important in the Station.
*/

/*! \fn extern void start_ad(enum admodtype adm);
    \brief starts the ADC of some given value.

    Starts the measure of adm. The normal measures should be ready in two cycles of Timer0. If the ADC is used already it will beep 5 times. This function will set admodus.
*/
/*! \fn extern void start_radar();
    \brief starts measuring the distance by "radar".

    This will enter the radar mode. In this time nothing else will be computated. It will last at maximum 0.15s (depending on the distance). The result is written in radarcont. 0 for near, 1000 for far away or error.
*/
/*! \def RESETP
    \brief used to stop the robot instantly. It can loose the right heading by this.
*/
/*! \def SETMODUS(nmodus)
    \brief Stops the Robot, sets the modus to nmodus and resets the modustimer (important for the new modus)
*/
/*! \def DIPPOS
    \brief gives back the status of the dip switches. (adds the values written on in). Values are 0 .. 3
*/

/*! \def POS360
    \brief the difference of values from the angle sensor that gives a 360 turn.
*/

/*! \def MINSPEED
    \brief the minimum speed that is given to the motors. This saves some power because the motors would not move anyway.
*/

/*! \enum admodus
    \brief Holds the value of the acutal ad-conversation
*/
/*! \var irbittime
    \brief the time for one bit in 36kHz cycles.
*/
/*! \var irdatatype
    \brief type to store the information from IR

    The bits are written/read from left to right. A '0' means signal 'off' a '1' means 'on'. The timing is determined by irbittime and the 36kHz clock from modulation. The first byte is the length of the string (including it self). So the string "04 aa 00 aa ff" wouldn't send the last FF!
*/

/*! \var time
    \brief the global timer.

    Is incremented by every "normal" Timer0 overflows ("normal" is everything besides ir-trans. and rec. and radar). The do_* functions can use it but should not change. It goes from 0 to 2^16.
*/

/*! \var modustime
    \brief a helping variable vor the do_functions. Mainly for building a state machine (is initialised at 0)
*/


/*! \var irstation_value
    \brief the read ir density from the loading station
*/
/*! \var light_value
    \brief the read light value.
*/
/*! \var vcc_value
   \brief the read voltage at the processor.

   It is done by comparing this to a zener diode and is highly nonlinear.
*/
/*! \var vcco_value
   \brief the read voltage at the bumper. Also influenced by the DIPPOS-2.
*/
/*! \var irpos_value
   \brief the distance to the next object measured by IR. Goes to about 30 cm.
*/
/*! \var radar_value
   \brief the distance to the next object measured by radar.

   From 50cm to 5m. a difference of one is about 1,96 cm.
*/

/*! \var heading
   \brief the actual heading of the robot.

   Goes from 0 to POS360. Is calculated from positionl and positionr in decode_position(). counterclockwise
*/
/*! \var positionl
   \brief the position of the left position sensor at the wheel.

   it is reset when positionl or positionr reaches 5000 so don't build your code at them.
*/

/*! \var positionr
   \brief the position of the right position sensor at the wheel.

   it is reset when positionl or positionr reaches 5000 so don't build your code at them.
*/
/*! \var speed
    \brief the speed to drive.

    When setting speed the robot drives forward or backward. The %direction is set with direction.
    Direction = +-64 is meant that one wheel is standing still. If the heading cannot be hold the speed is internally reduced.
    When the speed is very small the robot will not drive constantly it holds the position averagly.
    Direction = +-128 turns around the own axis. Speed and direction should only get values from -128 to 127.
*/

/*! \var direction
    \brief see speed.
*/

/*! \var l_to
    \brief the positionl to be reached

    Don't change directly but with drive_to(), at it will change by overflow of positionl and positionr.
*/

/*! \var r_to
    \brief the positionr to be reached

    Don't change directly but with drive_to(), at it will hcange by overflow of positionl and positionr.
*/
/*! \var stringvalid
   \brief Is set 1 if a valid string is received. Set it to 0 if computated. But also reset irrstring[0]!
*/
/*! \var soundplaying
    \brief if the sound is playing this variable is set to 1
*/
/*! \var sendready
   \brief This variable is used to wait for the IR-String send routines. It's used in WAITIR.
*/
/*! \var d_pos
   \brief the difference from the actual to the optimal position. in ticks like the POS360.
*/
/*! \var needpower
   \brief is set in the timer0 routine if the Battery is low.
*/